import os
from setuptools import setup
from Cython.Build import cythonize

dir_path = os.path.dirname(os.path.realpath(__file__))

setup(
    ext_modules=cythonize(
        os.path.join(dir_path, "_adaoptc.pyx"),
        compiler_directives={"language_level": "3"},
    )
)



import numpy as np
import platform
import warnings
import pandas as pd
from sklearn.base import BaseEstimator
from sklearn.base import RegressorMixin
from sklearn.preprocessing import PolynomialFeatures

try:
    from . import _boosterc as boosterc
except ImportError:
    import _boosterc as boosterc
from ..predictioninterval import PredictionInterval
from ..utils import cluster


class LSBoostRegressor(BaseEstimator, RegressorMixin):
    """LSBoost regressor.

    Attributes:

        n_estimators: int
            number of boosting iterations.

        learning_rate: float
            controls the learning speed at training time.

        n_hidden_features: int
            number of nodes in successive hidden layers.

        reg_lambda: float
            L2 regularization parameter for successive errors in the optimizer
            (at training time).

        alpha: float
            compromise between L1 and L2 regularization (must be in [0, 1]),
            for `solver` == 'enet'

        row_sample: float
            percentage of rows chosen from the training set.

        col_sample: float
            percentage of columns chosen from the training set.

        dropout: float
            percentage of nodes dropped from the training set.

        tolerance: float
            controls early stopping in gradient descent (at training time).

        direct_link: bool
            indicates whether the original features are included (True) in model's
            fitting or not (False).

        verbose: int
            progress bar (yes = 1) or not (no = 0) (currently).

        seed: int
            reproducibility seed for nodes_sim=='uniform', clustering and dropout.

        backend: str
            type of backend; must be in ('cpu', 'gpu', 'tpu')

        solver: str
            type of 'weak' learner; currently in ('ridge', 'lasso')

        activation: str
            activation function: currently 'relu', 'relu6', 'sigmoid', 'tanh'

        type_pi: str.
            type of prediction interval; currently "kde" (default) or "bootstrap".
            Used only in `self.predict`, for `self.replications` > 0 and `self.kernel`
            in ('gaussian', 'tophat'). Default is `None`.

        replications: int.
            number of replications (if needed) for predictive simulation.
            Used only in `self.predict`, for `self.kernel` in ('gaussian',
            'tophat') and `self.type_pi = 'kde'`. Default is `None`.

        n_clusters: int
            number of clusters for clustering the features

        clustering_method: str
            clustering method: currently 'kmeans', 'gmm'

        cluster_scaling: str
            scaling method for clustering: currently 'standard', 'robust', 'minmax'

        degree: int
            degree of features interactions to include in the model

        weights_distr: str
            distribution of weights for constructing the model's hidden layer;
            either 'uniform' or 'gaussian'

    """

    def __init__(
        self,
        n_estimators=100,
        learning_rate=0.1,
        n_hidden_features=5,
        reg_lambda=0.1,
        alpha=0.5,
        row_sample=1,
        col_sample=1,
        dropout=0,
        tolerance=1e-4,
        direct_link=1,
        verbose=1,
        seed=123,
        backend="cpu",
        solver="ridge",
        activation="relu",
        type_pi=None,
        replications=None,
        kernel=None,
        n_clusters=0,
        clustering_method="kmeans",
        cluster_scaling="standard",
        degree=None,
        weights_distr="uniform",
    ):
        if n_clusters > 0:
            assert clustering_method in (
                "kmeans",
                "gmm",
            ), "`clustering_method` must be in ('kmeans', 'gmm')"
            assert cluster_scaling in (
                "standard",
                "robust",
                "minmax",
            ), "`cluster_scaling` must be in ('standard', 'robust', 'minmax')"

        assert backend in (
            "cpu",
            "gpu",
            "tpu",
        ), "`backend` must be in ('cpu', 'gpu', 'tpu')"

        assert solver in (
            "ridge",
            "lasso",
            "enet",
        ), "`solver` must be in ('ridge', 'lasso', 'enet')"

        sys_platform = platform.system()

        if (sys_platform == "Windows") and (backend in ("gpu", "tpu")):
            warnings.warn(
                "No GPU/TPU computing on Windows yet, backend set to 'cpu'"
            )
            backend = "cpu"

        self.n_estimators = n_estimators
        self.learning_rate = learning_rate
        self.n_hidden_features = n_hidden_features
        self.reg_lambda = reg_lambda
        assert alpha >= 0 and alpha <= 1, "`alpha` must be in [0, 1]"
        self.alpha = alpha
        self.row_sample = row_sample
        self.col_sample = col_sample
        self.dropout = dropout
        self.tolerance = tolerance
        self.direct_link = direct_link
        self.verbose = verbose
        self.seed = seed
        self.backend = backend
        self.obj = None
        self.solver = solver
        self.activation = activation
        self.type_pi = type_pi
        self.replications = replications
        self.kernel = kernel
        self.n_clusters = n_clusters
        self.clustering_method = clustering_method
        self.cluster_scaling = cluster_scaling
        self.scaler_, self.label_encoder_, self.clusterer_ = None, None, None
        self.degree = degree
        self.poly_ = None
        self.weights_distr = weights_distr

    def fit(self, X, y, **kwargs):
        """Fit Booster (regressor) to training data (X, y)

        Args:

            X: {array-like}, shape = [n_samples, n_features]
                Training vectors, where n_samples is the number
                of samples and n_features is the number of features.

            y: array-like, shape = [n_samples]
               Target values.

            **kwargs: additional parameters to be passed to self.cook_training_set.

        Returns:

            self: object.
        """

        if isinstance(X, pd.DataFrame):
            X = X.values

        if self.degree is not None:
            assert isinstance(self.degree, int), "`degree` must be an integer"
            self.poly_ = PolynomialFeatures(
                degree=self.degree, interaction_only=True, include_bias=False
            )
            X = self.poly_.fit_transform(X)

        if self.n_clusters > 0:
            clustered_X, self.scaler_, self.label_encoder_, self.clusterer_ = (
                cluster(
                    X,
                    n_clusters=self.n_clusters,
                    method=self.clustering_method,
                    type_scaling=self.cluster_scaling,
                    training=True,
                    seed=self.seed,
                )
            )
            X = np.column_stack((X, clustered_X))

        try:
            self.obj = boosterc.fit_booster_regressor(
                X=np.asarray(X, order="C"),
                y=np.asarray(y, order="C"),
                n_estimators=self.n_estimators,
                learning_rate=self.learning_rate,
                n_hidden_features=self.n_hidden_features,
                reg_lambda=self.reg_lambda,
                alpha=self.alpha,
                row_sample=self.row_sample,
                col_sample=self.col_sample,
                dropout=self.dropout,
                tolerance=self.tolerance,
                direct_link=self.direct_link,
                verbose=self.verbose,
                seed=self.seed,
                backend=self.backend,
                solver=self.solver,
                activation=self.activation,
            )
        except ValueError:
            self.obj = _boosterc.fit_booster_regressor(
                X=np.asarray(X, order="C"),
                y=np.asarray(y, order="C"),
                n_estimators=self.n_estimators,
                learning_rate=self.learning_rate,
                n_hidden_features=self.n_hidden_features,
                reg_lambda=self.reg_lambda,
                alpha=self.alpha,
                row_sample=self.row_sample,
                col_sample=self.col_sample,
                dropout=self.dropout,
                tolerance=self.tolerance,
                direct_link=self.direct_link,
                verbose=self.verbose,
                seed=self.seed,
                backend=self.backend,
                solver=self.solver,
                activation=self.activation,
            )

        self.n_estimators = self.obj["n_estimators"]

        self.X_ = X

        self.y_ = y

        return self

    def predict(self, X, level=95, method=None, **kwargs):
        """Predict probabilities for test data X.

        Args:

            X: {array-like}, shape = [n_samples, n_features]
                Training vectors, where n_samples is the number
                of samples and n_features is the number of features.

            level: int
                Level of confidence (default = 95)

            method: str
                `None`, or 'splitconformal', 'localconformal'
                prediction (if you specify `return_pi = True`)

            **kwargs: additional parameters to be passed to
                self.cook_test_set

        Returns:

            probability estimates for test data: {array-like}
        """

        if isinstance(X, pd.DataFrame):
            X = X.values

        if self.degree is not None:
            X = self.poly_.transform(X)

        if self.n_clusters > 0:
            X = np.column_stack(
                (
                    X,
                    cluster(
                        X,
                        training=False,
                        scaler=self.scaler_,
                        label_encoder=self.label_encoder_,
                        clusterer=self.clusterer_,
                        seed=self.seed,
                    ),
                )
            )
        if "return_pi" in kwargs:
            assert method in (
                "splitconformal",
                "localconformal",
            ), "method must be in ('splitconformal', 'localconformal')"
            self.pi = PredictionInterval(
                obj=self,
                method=method,
                level=level,
                type_pi=self.type_pi,
                replications=self.replications,
                kernel=self.kernel,
            )
            self.pi.fit(self.X_, self.y_)
            self.X_ = None
            self.y_ = None
            preds = self.pi.predict(X, return_pi=True)
            return preds

        try:
            return boosterc.predict_booster_regressor(
                self.obj, np.asarray(X, order="C")
            )
        except ValueError:
            return _boosterc.predict_booster_regressor(
                self.obj, np.asarray(X, order="C")
            )



import numpy as np
import matplotlib.pyplot as plt
from collections import namedtuple
from sklearn.datasets import load_diabetes
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import train_test_split
from time import time


def get_beta(X, y):
    return np.linalg.solve(X.T @ X, X.T @ y)


def soft_thresholding(z, gamma):
    return np.sign(z) * np.maximum(np.abs(z) - gamma, 0)


def l1_norm(x):
    return np.sum(np.abs(x))


def fit_elasticnet(X_train, y_train, lam=1, alpha=0.5, tol=1e-5, max_iter=50):

    lam_alpha = lam * alpha

    scaler = StandardScaler(with_mean=True, with_std=True)

    scaled_X_train = scaler.fit_transform(X_train, y_train)

    y_train_mean = y_train.mean()

    centered_y_train = y_train - y_train_mean

    beta_prev = get_beta(scaled_X_train, centered_y_train)  # /!\ this is key

    p_train = X_train.shape[1]
    beta_prevs = np.repeat(beta_prev, repeats=X_train.shape[1]).reshape(
        p_train, p_train
    )
    np.fill_diagonal(beta_prevs, 0)

    centered_y_train_tilde = scaled_X_train @ beta_prevs

    weighted_residuals = np.mean(
        scaled_X_train * centered_y_train[:, None] - centered_y_train_tilde,
        axis=0,
    )

    beta = soft_thresholding(weighted_residuals, gamma=lam_alpha) / (
        1 + lam_alpha
    )

    converged = l1_norm(beta_prev - beta) < tol

    iteration = 0

    while converged == False and iteration < max_iter:
        beta_prev = beta
        beta_prevs = np.repeat(beta_prev, repeats=p_train).reshape(
            p_train, p_train
        )
        np.fill_diagonal(beta_prevs, 0)
        centered_y_train_tilde = scaled_X_train @ beta_prevs
        weighted_residuals = np.mean(
            scaled_X_train * centered_y_train[:, None] - centered_y_train_tilde,
            axis=0,
        )
        beta = soft_thresholding(weighted_residuals, gamma=lam_alpha) / (
            1 + lam_alpha
        )
        tol_ = l1_norm(beta_prev - beta)
        converged = tol_ < tol
        iteration += 1

    DescribeResult = namedtuple(
        "DescribeResult",
        ("coef_", "iteration_", "tol_", "converged", "y_train_mean", "scaler"),
    )

    return DescribeResult(
        beta, iteration, tol_, converged, y_train_mean, scaler
    )


def predict_elasticnet(X_test, fitted_elasticnet):
    return (
        fitted_elasticnet.y_train_mean
        + fitted_elasticnet.scaler.transform(X_test) @ fitted_elasticnet.coef_
    )



import cython 
import numpy as np 
from typing import Any, Callable, Iterable
from math import factorial
from itertools import chain, combinations


@cython.cclass
class ShapExplainer():
    def __init__(self,
                 model: Callable[[np.ndarray], cython.float], 
                 background_dataset: np.ndarray,
                 max_samples: cython.int = -1):
        self.model = model

        # Ensure that background_dataset is valid and has a positive size
        if background_dataset.shape[0] == 0:
            raise ValueError("background_dataset cannot be empty")

        # If max_samples is -1, we use the full background dataset
        if max_samples == -1 or max_samples >= background_dataset.shape[0]:
            self.background_dataset = background_dataset
        else:
            # Ensure max_samples is non-negative and within the valid range
            if max_samples < 0:
                raise ValueError("max_samples must be non-negative")
            
            # Subsample the background dataset
            rng = np.random.default_rng()
            self.background_dataset = rng.choice(background_dataset, size=max_samples, replace=False)

    @cython.ccall
    def shap_values(self, X: np.ndarray): #-> np.ndarray:
        "SHAP Values for instances in DataFrame or 2D array"
        i: cython.int               # cdef int i
        j: cython.int               # cdef int i
        shap_values = np.empty(X.shape)
        for i in range(X.shape[0]):
            for j in range(X.shape[1]):
                shap_values[i, j] = self._compute_single_shap_value(j, X[i, :])
        return shap_values

    @cython.cfunc      
    def _compute_single_shap_value(self, 
                                   feature: cython.int,
                                   instance: np.array) -> cython.float:
        "Compute a single SHAP value (equation 4)"
        n_features: cython.int               # cdef int n_features
        shap_value: cython.float             # cdef float shap_value
        n_features = len(instance)
        shap_value = 0
        for subset in self._get_all_other_feature_subsets(n_features, feature):
            n_subset = len(subset)
            prediction_without_feature = self._subset_model_approximation(
                subset, 
                instance
            )
            prediction_with_feature = self._subset_model_approximation(
                subset + (feature,), 
                instance
            )
            factor = self._permutation_factor(n_features, n_subset)
            shap_value += factor * (prediction_with_feature - prediction_without_feature)
        return shap_value
    
    def _get_all_subsets(self, items: list) -> Iterable:
        r: cython.int               # cdef int r
        return chain.from_iterable(combinations(items, r) for r in range(len(items)+1))
    
    def _get_all_other_feature_subsets(self, n_features: cython.int, feature_of_interest: cython.int):
        return self._get_all_subsets(np.delete(np.arange(n_features), feature_of_interest).tolist())

    def _permutation_factor(self, n_features: cython.int, n_subset: cython.int):
        return (
            factorial(n_subset) 
            * factorial(n_features - n_subset - 1) 
            / factorial(n_features) 
        )
    
    def _subset_model_approximation(self, 
                                    feature_subset: tuple[cython.int, ...], 
                                    instance: np.array) -> cython.float:
        masked_background_dataset = self.background_dataset.copy()
        # https://mathspp.com/blog/breaking-out-of-nested-loops-with-generators
        j: cython.int               # cdef int j
        for j in range(masked_background_dataset.shape[1]):
            if j in feature_subset:
                masked_background_dataset[:, j] = instance[j]
        conditional_expectation_of_model = np.mean(
            self.model(masked_background_dataset)
        )
        return conditional_expectation_of_model          


from __future__ import division
import numpy as np


def calc_p(ncal, ngt, neq, smoothing=False):
    if smoothing:
        return (ngt + (neq + 1) * np.random.uniform(0, 1)) / (ncal + 1)
    else:
        return (ngt + neq + 1) / (ncal + 1)



from locale import normalize
import numpy as np
import pickle
from collections import namedtuple
from sklearn.base import BaseEstimator, RegressorMixin
from sklearn.model_selection import train_test_split
from sklearn.ensemble import ExtraTreesRegressor
from sklearn.preprocessing import StandardScaler
from sklearn.neighbors import KernelDensity
from sklearn.model_selection import GridSearchCV
from scipy.stats import gaussian_kde
from tqdm import tqdm
from ..nonconformist import IcpRegressor
from ..nonconformist import RegressorNc
from ..nonconformist import RegressorNormalizer, AbsErrorErrFunc
from ..utils import Progbar


class PredictionInterval(BaseEstimator, RegressorMixin):
    """Class PredictionInterval: Obtain prediction intervals.

    Attributes:

        obj: an object;
            fitted object containing methods `fit` and `predict`

        method: a string;
            method for constructing the prediction intervals.
            Currently "splitconformal" (default) and "localconformal"

        level: a float;
            Confidence level for prediction intervals. Default is 95,
            equivalent to a miscoverage error of 5 (%)

        replications: an integer;
            Number of replications for simulated conformal (default is `None`)

        type_pi: a string;
            type of prediction interval: currently "kde" (default) or "bootstrap"

        seed: an integer;
            Reproducibility of fit (there's a random split between fitting and calibration data)
    """

    def __init__(
        self,
        obj,
        method="splitconformal",
        level=95,
        type_pi="bootstrap",
        replications=None,
        kernel=None,
        agg="mean",
        seed=123,
    ):

        self.obj = obj
        self.method = method
        self.level = level
        self.type_pi = type_pi
        self.replications = replications
        self.kernel = kernel
        self.agg = agg
        self.seed = seed
        self.alpha_ = 1 - self.level / 100
        self.quantile_ = None
        self.icp_ = None
        self.calibrated_residuals_ = None
        self.scaled_calibrated_residuals_ = None
        self.calibrated_residuals_scaler_ = None
        self.kde_ = None

    def fit(self, X, y):
        """Fit the `method` to training data (X, y).

        Args:

            X: array-like, shape = [n_samples, n_features];
                Training set vectors, where n_samples is the number
                of samples and n_features is the number of features.

            y: array-like, shape = [n_samples, ]; Target values.

        """

        X_train, X_calibration, y_train, y_calibration = train_test_split(
            X, y, test_size=0.5, random_state=self.seed
        )

        if self.method == "splitconformal":

            self.obj.fit(X_train, y_train)
            preds_calibration = self.obj.predict(X_calibration)
            self.calibrated_residuals_ = y_calibration - preds_calibration
            absolute_residuals = np.abs(self.calibrated_residuals_)
            self.calibrated_residuals_scaler_ = StandardScaler(
                with_mean=True, with_std=True
            )
            self.scaled_calibrated_residuals_ = (
                self.calibrated_residuals_scaler_.fit_transform(
                    self.calibrated_residuals_.reshape(-1, 1)
                ).ravel()
            )
            try:
                # numpy version >= 1.22
                self.quantile_ = np.quantile(
                    a=absolute_residuals, q=self.level / 100, method="higher"
                )
            except:
                # numpy version < 1.22
                self.quantile_ = np.quantile(
                    a=absolute_residuals,
                    q=self.level / 100,
                    interpolation="higher",
                )

        if self.method == "localconformal":
            raise NotImplementedError(
                "Method `localconformal` is not implemented yet."
            )
            mad_estimator = ExtraTreesRegressor()
            normalizer = RegressorNormalizer(
                self.obj, mad_estimator, AbsErrorErrFunc()
            )
            nc = RegressorNc(self.obj, AbsErrorErrFunc(), normalizer)
            self.icp_ = IcpRegressor(nc)
            self.icp_.fit(X_train, y_train)
            self.icp_.calibrate(X_calibration, y_calibration)
            print(f"X_train: {X_train}")
            print(f"X_train.shape: {X_train.shape}")
            print(f"y_train: {y_train}")
            print(f"X_calibration: {X_calibration}")
            print(f"X_calibration.shape: {X_calibration.shape}")
            print(f"y_calibration: {y_calibration}")
            print(f"self.icp_: {self.icp_}")
            preds_calibration = self.icp_.predict(
                X_calibration
            )  # it's a tensor
            print(f"preds_calibration: {preds_calibration}")
            print(f"preds_calibration.shape: {preds_calibration.shape}")
            self.calibrated_residuals_ = y_calibration - preds_calibration
            absolute_residuals = np.abs(self.calibrated_residuals_)
            self.calibrated_residuals_scaler_ = StandardScaler(
                with_mean=True, with_std=True
            )
            self.scaled_calibrated_residuals_ = (
                self.calibrated_residuals_scaler_.fit_transform(
                    self.calibrated_residuals_.reshape(-1, 1)
                ).ravel()
            )

        return self

    def predict(self, X, return_pi=False):
        """Obtain predictions and prediction intervals

        Args:

            X: array-like, shape = [n_samples, n_features];
                Testing set vectors, where n_samples is the number
                of samples and n_features is the number of features.

            return_pi: boolean
                Whether the prediction interval is returned or not.
                Default is False, for compatibility with other _estimators_.
                If True, a tuple containing the predictions + lower and upper
                bounds is returned.

        """

        pred = self.obj.predict(X)

        if self.method == "splitconformal":

            if self.replications is None:

                if return_pi:

                    DescribeResult = namedtuple(
                        "DescribeResult", ("mean", "lower", "upper")
                    )

                    return DescribeResult(
                        pred, pred - self.quantile_, pred + self.quantile_
                    )

                else:

                    return pred

            else:  # if self.replications is not None

                assert self.type_pi in (
                    "bootstrap",
                    "kde",
                ), "`self.type_pi` must be in ('bootstrap', 'kde')"

                if self.type_pi == "bootstrap":
                    np.random.seed(self.seed)
                    self.residuals_sims_ = np.asarray(
                        [
                            np.random.choice(
                                a=self.scaled_calibrated_residuals_,
                                size=X.shape[0],
                            )
                            for _ in range(self.replications)
                        ]
                    ).T
                    self.sims_ = np.asarray(
                        [
                            pred
                            + self.calibrated_residuals_scaler_.scale_[0]
                            * self.residuals_sims_[:, i].ravel()
                            for i in tqdm(range(self.replications))
                        ]
                    ).T
                elif self.type_pi == "kde":
                    self.kde_ = gaussian_kde(
                        dataset=self.scaled_calibrated_residuals_
                    )
                    self.sims_ = np.asarray(
                        [
                            pred
                            + self.calibrated_residuals_scaler_.scale_[0]
                            * self.kde_.resample(
                                size=X.shape[0], seed=self.seed + i
                            ).ravel()
                            for i in tqdm(range(self.replications))
                        ]
                    ).T

                self.mean_ = np.mean(self.sims_, axis=1)
                self.lower_ = np.quantile(
                    self.sims_, q=self.alpha_ / 200, axis=1
                )
                self.upper_ = np.quantile(
                    self.sims_, q=1 - self.alpha_ / 200, axis=1
                )

                DescribeResult = namedtuple(
                    "DescribeResult", ("mean", "sims", "lower", "upper")
                )

                return DescribeResult(
                    self.mean_, self.sims_, self.lower_, self.upper_
                )

        if self.method == "localconformal":

            if self.replications is None:

                if return_pi:

                    predictions_bounds = self.icp_.predict(
                        X, significance=1 - self.level
                    )
                    DescribeResult = namedtuple(
                        "DescribeResult", ("mean", "lower", "upper")
                    )
                    return DescribeResult(
                        pred, predictions_bounds[:, 0], predictions_bounds[:, 1]
                    )

                else:

                    return pred

            else:  # if self.replications is not None

                assert self.type_pi in (
                    "bootstrap",
                    "kde",
                ), "`self.type_pi` must be in ('bootstrap', 'kde')"

                if self.type_pi == "bootstrap":
                    np.random.seed(self.seed)
                    self.residuals_sims_ = np.asarray(
                        [
                            np.random.choice(
                                a=self.scaled_calibrated_residuals_,
                                size=X.shape[0],
                            )
                            for _ in range(self.replications)
                        ]
                    ).T
                    self.sims_ = np.asarray(
                        [
                            pred
                            + self.calibrated_residuals_scaler_.scale_[0]
                            * self.residuals_sims_[:, i].ravel()
                            for i in tqdm(range(self.replications))
                        ]
                    ).T
                elif self.type_pi == "kde":
                    self.kde_ = gaussian_kde(
                        dataset=self.scaled_calibrated_residuals_
                    )
                    self.sims_ = np.asarray(
                        [
                            pred
                            + self.calibrated_residuals_scaler_.scale_[0]
                            * self.kde_.resample(
                                size=X.shape[0], seed=self.seed + i
                            ).ravel()
                            for i in tqdm(range(self.replications))
                        ]
                    ).T

                self.mean_ = np.mean(self.sims_, axis=1)
                self.lower_ = np.quantile(
                    self.sims_, q=self.alpha_ / 200, axis=1
                )
                self.upper_ = np.quantile(
                    self.sims_, q=1 - self.alpha_ / 200, axis=1
                )

                DescribeResult = namedtuple(
                    "DescribeResult", ("mean", "sims", "lower", "upper")
                )

                return DescribeResult(
                    self.mean_, self.sims_, self.lower_, self.upper_
                )



import sys
import logging
import os

from ._config import get_config, set_config, config_context

logger = logging.getLogger(__name__)


# PEP0440 compatible formatted version, see:
# https://www.python.org/dev/peps/pep-0440/
#
# Generic release markers:
#   X.Y
#   X.Y.Z   # For bugfix releases
#
# Admissible pre-release markers:
#   X.YaN   # Alpha release
#   X.YbN   # Beta release
#   X.YrcN  # Release Candidate
#   X.Y     # Final release
#
# Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
# 'X.Y.dev0' is the canonical version of 'X.Y.dev'
#
# __version__ = "0.10.0"


# On OSX, we can get a runtime error due to multiple OpenMP libraries loaded
# simultaneously. This can happen for instance when calling BLAS inside a
# prange. Setting the following environment variable allows multiple OpenMP
# libraries to be loaded. It should not degrade performances since we manually
# take care of potential over-subcription performance issues, in sections of
# the code where nested OpenMP loops can happen, by dynamically reconfiguring
# the inner OpenMP runtime to temporarily disable it while under the scope of
# the outer OpenMP parallel section.
os.environ.setdefault("KMP_DUPLICATE_LIB_OK", "True")

# Workaround issue discovered in intel-openmp 2019.5:
# https://github.com/ContinuumIO/anaconda-issues/issues/11294
os.environ.setdefault("KMP_INIT_AT_FORK", "FALSE")

try:
    # This variable is injected in the __builtins__ by the build
    # process. It is used to enable importing subpackages of mlsauce when
    # the binaries are not built
    # mypy error: Cannot determine type of '__MLSAUCE_SETUP__'
    __MLSAUCE_SETUP__  # type: ignore
except NameError:
    __MLSAUCE_SETUP__ = False

if __MLSAUCE_SETUP__:
    sys.stderr.write("Partial import of mlsauce during the build process.\n")
    # We are not importing the rest of scikit-learn during the build
    # process, as it may not be compiled yet

else:
    from .adaopt import AdaOpt
    from .booster import LSBoostClassifier, LSBoostRegressor
    from .datasets import download
    from .elasticnet import ElasticNetRegressor
    from .lasso import LassoRegressor
    from .ridge import RidgeRegressor
    from .shapley import ShapExplainer
    from .stump import StumpClassifier    

    # from .encoders import corrtarget_encoder

    __all__ = [
        "AdaOpt",
        "LSBoostClassifier",
        "StumpClassifier",
        "ElasticNetRegressor",
        "LassoRegressor",
        "LSBoostRegressor",
        "RidgeRegressor",
        "ShapExplainer",
        # Other imports
        # "corrtarget_encoder",
        "download",
        # Non-modules:
        "get_config",
        "set_config",
        "config_context",
    ]


def setup_module(module):
    """Fixture for the tests to assure globally controllable seeding of RNGs"""
    import os
    import numpy as np
    import random

    # Check if a random seed exists in the environment, if not create one.
    _random_seed = os.environ.get("MLSAUCE_SEED", None)
    if _random_seed is None:
        _random_seed = np.random.uniform() * np.iinfo(np.int32).max
    _random_seed = int(_random_seed)
    print("I: Seeding RNGs with %r" % _random_seed)
    np.random.seed(_random_seed)
    random.seed(_random_seed)



#!/usr/bin/env python

import os
import platform
import sys
import subprocess
from setuptools import Extension, find_packages, setup

subprocess.check_call([sys.executable, "-m", "pip", "install", "numpy"])
subprocess.check_call([sys.executable, "-m", "pip", "install", "cython"])

from Cython.Build import cythonize
import numpy

DISTNAME = 'mlsauce'
DESCRIPTION = 'Miscellaneous Statistical/Machine Learning tools'
LONG_DESCRIPTION = 'Miscellaneous Statistical/Machine Learning tools'
MAINTAINER = 'T. Moudiki'
MAINTAINER_EMAIL = 'thierry.moudiki@gmail.com'
LICENSE = 'BSD3 Clause Clear'
VERSION = '0.20.0'

# Minimum versions for dependencies
if platform.python_implementation() == 'PyPy':
    SCIPY_MIN_VERSION = '1.1.0'
    NUMPY_MIN_VERSION = '1.14.0'
else:
    SCIPY_MIN_VERSION = '0.19.0'
    NUMPY_MIN_VERSION = '1.13.0'

install_requires = [
    f"numpy >= {NUMPY_MIN_VERSION}",
    f"scipy >= {SCIPY_MIN_VERSION}",
    "joblib >= 1.2.0",
    "scikit-learn >= 0.18.0",
    "threadpoolctl >= 2.0.0",
    "pandas >= 0.25.3",
    "requests >= 2.31.0"
]

# Include JAX only on Linux and macOS
if platform.system() in ('Linux', 'Darwin'):
    install_requires += ["jax >= 0.1.72", "jaxlib >= 0.1.51"]

# Define extensions
ext_modules = [
    Extension(
        "mlsauce.adaopt._adaoptc",
        sources=["mlsauce/adaopt/_adaoptc.pyx"],
        include_dirs=[numpy.get_include()]
    ),
    Extension(
        "mlsauce.booster._boosterc",
        sources=["mlsauce/booster/_boosterc.pyx"],
        include_dirs=[numpy.get_include()]
    ),
    Extension(
        "mlsauce.lasso._lassoc",
        sources=["mlsauce/lasso/_lassoc.pyx"],
        include_dirs=[numpy.get_include()]
    ),
    Extension(
        "mlsauce.ridge._ridgec",
        sources=["mlsauce/ridge/_ridgec.pyx"],
        include_dirs=[numpy.get_include()]
    ),
    Extension(
        "mlsauce.stump._stumpc",
        sources=["mlsauce/stump/_stumpc.pyx"],
        include_dirs=[numpy.get_include()]
    ),
]

def setup_package():
    metadata = dict(
        name=DISTNAME,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        license=LICENSE,
        version=VERSION,
        long_description=LONG_DESCRIPTION,
        classifiers=[
            'Development Status :: 2 - Pre-Alpha',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
        ],
        platforms=["linux", "macosx", "windows"],
        python_requires=">=3.5",
        install_requires=install_requires,
        setup_requires=["numpy>=1.13.0"],
        packages=find_packages(),
        ext_modules=cythonize(ext_modules),
        zip_safe=False,  # the package can run out of an .egg file
        include_package_data=True,
    )
    
    setup(**metadata)

if __name__ == "__main__":
    setup_package()
